@@ -106,10 +106,14 @@ struct _PhotosApplication
   GSimpleAction *search_type_action;
   GSimpleAction *sel_all_action;
   GSimpleAction *sel_none_action;
+  GSimpleAction *selection_mode_action;
   GSimpleAction *set_bg_action;
   GSimpleAction *set_ss_action;
   GSimpleAction *share_action;
   GSimpleAction *sharpen_action;
+  GSimpleAction *zoom_best_fit_action;
+  GSimpleAction *zoom_in_action;
+  GSimpleAction *zoom_out_action;
   GtkWidget *main_window;
   PhotosBaseManager *shr_pnt_mngr;
   PhotosCameraCache *camera_cache;
@@ -162,11 +166,13 @@ static const gchar *REQUIRED_GEGL_OPS[] =
   "gegl:exposure",
   "gegl:gray",
   "gegl:load",
+  "gegl:noise-reduction",
   "gegl:nop",
   "gegl:pixbuf",
   "gegl:rotate-on-center",
   "gegl:save-pixbuf",
   "gegl:scale-ratio",
+  "gegl:unsharp-mask",
 
   /* Used by gegl:load */
   "gegl:jpg-load",
@@ -254,7 +260,7 @@ photos_application_refresh_data_free (PhotosApplicationRefreshData *data)
 
 
 static void
-photos_application_help (PhotosApplication *self, GVariant *parameter)
+photos_application_help (PhotosApplication *self)
 {
   GtkWindow *parent;
   guint32 time;
@@ -266,7 +272,7 @@ photos_application_help (PhotosApplication *self, GVariant *parameter)
 
 
 static void
-photos_application_about (PhotosApplication *self, GVariant *parameter)
+photos_application_about (PhotosApplication *self)
 {
   photos_main_window_show_about (PHOTOS_MAIN_WINDOW (self->main_window));
 }
@@ -277,13 +283,15 @@ photos_application_action_toggle (GSimpleAction *simple, GVariant *parameter, gp
 {
   GVariant *state;
   GVariant *new_state;
+  gboolean state_value;
 
   state = g_action_get_state (G_ACTION (simple));
-  if (state == NULL)
-    return;
+  g_return_if_fail (state != NULL);
 
-  new_state = g_variant_new ("b", !g_variant_get_boolean (state));
+  state_value = g_variant_get_boolean (state);
+  new_state = g_variant_new ("b", !state_value);
   g_action_change_state (G_ACTION (simple), new_state);
+
   g_variant_unref (state);
 }
 
@@ -293,7 +301,7 @@ photos_application_get_selection_or_active_item (PhotosApplication *self)
 {
   PhotosBaseItem *item = NULL;
 
-  if (photos_selection_controller_get_selection_mode (self->sel_cntrlr))
+  if (photos_utils_get_selection_mode ())
     {
       GList *selection;
       const gchar *urn;
@@ -332,7 +340,10 @@ photos_application_actions_update (PhotosApplication *self)
   load_state = photos_item_manager_get_load_state (self->state->item_mngr);
   mode = photos_mode_controller_get_window_mode (self->state->mode_cntrlr);
   selection = photos_selection_controller_get_selection (self->sel_cntrlr);
-  selection_mode = photos_selection_controller_get_selection_mode (self->sel_cntrlr);
+  selection_mode = photos_utils_get_selection_mode ();
+
+  g_simple_action_set_enabled (self->zoom_best_fit_action, FALSE);
+  g_simple_action_set_enabled (self->zoom_out_action, FALSE);
 
   enable = (mode == PHOTOS_WINDOW_MODE_EDIT);
   g_simple_action_set_enabled (self->blacks_exposure_action, enable);
@@ -354,6 +365,7 @@ photos_application_actions_update (PhotosApplication *self)
   g_simple_action_set_enabled (self->search_type_action, enable);
   g_simple_action_set_enabled (self->sel_all_action, enable);
   g_simple_action_set_enabled (self->sel_none_action, enable);
+  g_simple_action_set_enabled (self->selection_mode_action, enable);
 
   enable = (mode == PHOTOS_WINDOW_MODE_PREVIEW);
   g_simple_action_set_enabled (self->load_next_action, enable);
@@ -363,6 +375,7 @@ photos_application_actions_update (PhotosApplication *self)
   g_simple_action_set_enabled (self->gear_action, enable);
   g_simple_action_set_enabled (self->set_bg_action, enable);
   g_simple_action_set_enabled (self->set_ss_action, enable);
+  g_simple_action_set_enabled (self->zoom_in_action, enable);
 
   enable = ((load_state == PHOTOS_LOAD_STATE_FINISHED && mode == PHOTOS_WINDOW_MODE_PREVIEW)
             || (selection_mode && item != NULL));
@@ -737,6 +750,8 @@ photos_application_activate_result (PhotosApplication *self,
 {
   GObject *item;
 
+  photos_debug (PHOTOS_DEBUG_APPLICATION, "PhotosApplication::activate_result");
+
   self->activation_timestamp = timestamp;
 
   item = photos_base_manager_get_object_by_id (self->state->item_mngr, identifier);
@@ -813,6 +828,8 @@ photos_application_edit_current (PhotosApplication *self)
   item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (self->state->item_mngr));
   g_return_if_fail (item != NULL);
 
+  g_action_activate (G_ACTION (self->zoom_best_fit_action), NULL);
+
   photos_base_item_pipeline_snapshot (item);
   photos_mode_controller_set_window_mode (self->state->mode_cntrlr, PHOTOS_WINDOW_MODE_EDIT);
 }
@@ -877,7 +894,7 @@ photos_application_edit_revert (PhotosApplication *self, GVariant *parameter)
 
 
 static void
-photos_application_fullscreen (PhotosApplication *self, GVariant *parameter)
+photos_application_fullscreen (PhotosApplication *self)
 {
   photos_mode_controller_toggle_fullscreen (self->state->mode_cntrlr);
 }
@@ -897,6 +914,8 @@ photos_application_launch_search (PhotosApplication *self, const gchar* const *t
   GVariant *state;
   gchar *str;
 
+  photos_debug (PHOTOS_DEBUG_APPLICATION, "PhotosApplication::launch_search");
+
   if (!photos_application_create_window (self))
     return;
 
@@ -925,11 +944,12 @@ static void
 photos_application_open_current (PhotosApplication *self)
 {
   PhotosBaseItem *item;
+  GVariant *new_state;
   guint32 time;
 
   time = gtk_get_current_event_time ();
 
-  if (photos_selection_controller_get_selection_mode (self->sel_cntrlr))
+  if (photos_utils_get_selection_mode ())
     {
       GList *l;
       GList *selection;
@@ -951,7 +971,8 @@ photos_application_open_current (PhotosApplication *self)
       photos_base_item_open (item, GTK_WINDOW (self->main_window), time);
     }
 
-  photos_selection_controller_set_selection_mode (self->sel_cntrlr, FALSE);
+  new_state = g_variant_new ("b", FALSE);
+  g_action_change_state (G_ACTION (self->selection_mode_action), new_state);
 }
 
 
@@ -1027,9 +1048,12 @@ static void
 photos_application_properties_response (GtkDialog *dialog, gint response_id, gpointer user_data)
 {
   PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+  GVariant *new_state;
 
   gtk_widget_destroy (GTK_WIDGET (dialog));
-  photos_selection_controller_set_selection_mode (self->sel_cntrlr, FALSE);
+
+  new_state = g_variant_new ("b", FALSE);
+  g_action_change_state (G_ACTION (self->selection_mode_action), new_state);
 }
 
 
@@ -1190,7 +1214,7 @@ photos_application_remote_display_current (PhotosApplication *self)
 
 
 static void
-photos_application_quit (PhotosApplication *self, GVariant *parameter)
+photos_application_quit (PhotosApplication *self)
 {
   gtk_window_close (GTK_WINDOW (self->main_window));
 }
@@ -1231,6 +1255,7 @@ photos_application_save_response (GtkDialog *dialog, gint response_id, gpointer
   GError *error;
   GFile *export = NULL;
   GFile *tmp;
+  GVariant *new_state;
   PhotosBaseItem *item;
   const gchar *export_dir_name;
   const gchar *pictures_path;
@@ -1243,7 +1268,8 @@ photos_application_save_response (GtkDialog *dialog, gint response_id, gpointer
   item = photos_application_get_selection_or_active_item (self);
   g_return_if_fail (item != NULL);
 
-  photos_selection_controller_set_selection_mode (self->sel_cntrlr, FALSE);
+  new_state = g_variant_new ("b", FALSE);
+  g_action_change_state (G_ACTION (self->selection_mode_action), new_state);
 
   pictures_path = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
   export_path = g_build_filename (pictures_path, PHOTOS_EXPORT_SUBPATH, NULL);
@@ -1383,6 +1409,7 @@ static void
 photos_application_share_response (GtkDialog *dialog, gint response_id, gpointer user_data)
 {
   PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+  GVariant *new_state;
   PhotosBaseItem *item;
   PhotosSharePoint *share_point;
 
@@ -1395,7 +1422,8 @@ photos_application_share_response (GtkDialog *dialog, gint response_id, gpointer
   item = photos_application_get_selection_or_active_item (self);
   g_return_if_fail (item != NULL);
 
-  photos_selection_controller_set_selection_mode (self->sel_cntrlr, FALSE);
+  new_state = g_variant_new ("b", FALSE);
+  g_action_change_state (G_ACTION (self->selection_mode_action), new_state);
 
   g_application_hold (G_APPLICATION (self));
   g_application_mark_busy (G_APPLICATION (self));
@@ -1528,11 +1556,31 @@ photos_application_selection_changed (PhotosApplication *self)
 }
 
 
+static void
+photos_application_selection_mode_notify_state (PhotosApplication *self)
+{
+  if (photos_utils_get_selection_mode ())
+    {
+      const gchar *selection_mode_accels[2] = {"Escape", NULL};
+      gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.selection-mode", selection_mode_accels);
+    }
+  else
+    {
+      const gchar *selection_mode_accels[1] = {NULL};
+      gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.selection-mode", selection_mode_accels);
+    }
+
+  photos_application_actions_update (self);
+}
+
+
 static void
 photos_application_activate (GApplication *application)
 {
   PhotosApplication *self = PHOTOS_APPLICATION (application);
 
+  photos_debug (PHOTOS_DEBUG_APPLICATION, "PhotosApplication::activate");
+
   if (self->main_window == NULL)
     {
       if (!photos_application_create_window (self))
@@ -1556,12 +1604,29 @@ photos_application_dbus_register (GApplication *application,
   gboolean ret_val = FALSE;
   gchar *search_provider_path = NULL;
 
+  photos_debug (PHOTOS_DEBUG_APPLICATION,
+                "PhotosApplication::dbus_register: object_path: %s, search_provider: %p",
+                object_path,
+                self->search_provider);
+
+  g_return_val_if_fail (self->search_provider == NULL, FALSE);
+
   if (!G_APPLICATION_CLASS (photos_application_parent_class)->dbus_register (application,
                                                                              connection,
                                                                              object_path,
                                                                              error))
     goto out;
 
+  self->search_provider = photos_search_provider_new ();
+  g_signal_connect_swapped (self->search_provider,
+                            "activate-result",
+                            G_CALLBACK (photos_application_activate_result),
+                            self);
+  g_signal_connect_swapped (self->search_provider,
+                            "launch-search",
+                            G_CALLBACK (photos_application_launch_search),
+                            self);
+
   search_provider_path = g_strconcat (object_path, PHOTOS_SEARCH_PROVIDER_PATH_SUFFIX, NULL);
   if (!photos_search_provider_dbus_export (self->search_provider, connection, search_provider_path, error))
     {
@@ -1572,6 +1637,10 @@ photos_application_dbus_register (GApplication *application,
   ret_val = TRUE;
 
  out:
+  photos_debug (PHOTOS_DEBUG_APPLICATION,
+                "PhotosApplication::dbus_register: Done: %d, search_provider: %p",
+                ret_val,
+                self->search_provider);
   g_free (search_provider_path);
   return ret_val;
 }
@@ -1584,6 +1653,11 @@ photos_application_dbus_unregister (GApplication *application,
 {
   PhotosApplication *self = PHOTOS_APPLICATION (application);
 
+  photos_debug (PHOTOS_DEBUG_APPLICATION,
+                "PhotosApplication::dbus_unregister: object_path: %s, search_provider: %p",
+                object_path,
+                self->search_provider);
+
   if (self->search_provider != NULL)
     {
       gchar *search_provider_path = NULL;
@@ -1595,6 +1669,10 @@ photos_application_dbus_unregister (GApplication *application,
     }
 
   G_APPLICATION_CLASS (photos_application_parent_class)->dbus_unregister (application, connection, object_path);
+
+  photos_debug (PHOTOS_DEBUG_APPLICATION,
+                "PhotosApplication::dbus_unregister: Done: search_provider: %p",
+                self->search_provider);
 }
 
 
@@ -1603,6 +1681,8 @@ photos_application_handle_local_options (GApplication *application, GVariantDict
 {
   gint ret_val = -1;
 
+  photos_debug (PHOTOS_DEBUG_APPLICATION, "PhotosApplication::handle_local_options");
+
   if (g_variant_dict_contains (options, "version"))
     {
       const gchar *version;
@@ -1622,6 +1702,8 @@ photos_application_shutdown (GApplication *application)
   PhotosApplication *self = PHOTOS_APPLICATION (application);
   guint refresh_miner_ids_size;
 
+  photos_debug (PHOTOS_DEBUG_APPLICATION, "PhotosApplication::shutdown");
+
   refresh_miner_ids_size = g_hash_table_size (self->refresh_miner_ids);
   g_assert (refresh_miner_ids_size == 0);
 
@@ -1658,6 +1740,11 @@ photos_application_startup (GApplication *application)
   const gchar *save_accels[2] = {"<Primary>x", NULL};
   const gchar *search_accels[2] = {"<Primary>f", NULL};
   const gchar *select_all_accels[2] = {"<Primary>a", NULL};
+  const gchar *zoom_best_fit_accels[3] = {"<Primary>0", NULL};
+  const gchar *zoom_in_accels[3] = {"<Primary>plus", "<Primary>equal", NULL};
+  const gchar *zoom_out_accels[2] = {"<Primary>minus", NULL};
+
+  photos_debug (PHOTOS_DEBUG_APPLICATION, "PhotosApplication::startup");
 
   G_APPLICATION_CLASS (photos_application_parent_class)->startup (application);
 
@@ -1715,11 +1802,6 @@ photos_application_startup (GApplication *application)
                             G_CALLBACK (photos_application_selection_changed),
                             self);
 
-  g_signal_connect_swapped (self->sel_cntrlr,
-                            "selection-mode-changed",
-                            G_CALLBACK (photos_application_selection_changed),
-                            self);
-
   action = g_simple_action_new ("about", NULL);
   g_signal_connect_swapped (action, "activate", G_CALLBACK (photos_application_about), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
@@ -1843,6 +1925,15 @@ photos_application_startup (GApplication *application)
   self->sel_none_action = g_simple_action_new ("select-none", NULL);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->sel_none_action));
 
+  state = g_variant_new ("b", FALSE);
+  self->selection_mode_action = g_simple_action_new_stateful ("selection-mode", NULL, state);
+  g_signal_connect (self->selection_mode_action, "activate", G_CALLBACK (photos_application_action_toggle), self);
+  g_signal_connect_swapped (self->selection_mode_action,
+                            "notify::state",
+                            G_CALLBACK (photos_application_selection_mode_notify_state),
+                            self);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->selection_mode_action));
+
   self->set_bg_action = g_simple_action_new ("set-background", NULL);
   g_object_set_data_full (G_OBJECT (self->set_bg_action),
                           "settings",
@@ -1866,6 +1957,15 @@ photos_application_startup (GApplication *application)
   self->sharpen_action = g_simple_action_new ("sharpen-current", G_VARIANT_TYPE_DOUBLE);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->sharpen_action));
 
+  self->zoom_best_fit_action = g_simple_action_new ("zoom-best-fit", NULL);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->zoom_best_fit_action));
+
+  self->zoom_in_action = g_simple_action_new ("zoom-in", G_VARIANT_TYPE_DOUBLE);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->zoom_in_action));
+
+  self->zoom_out_action = g_simple_action_new ("zoom-out", G_VARIANT_TYPE_DOUBLE);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (self->zoom_out_action));
+
   g_signal_connect_swapped (self->state->mode_cntrlr,
                             "window-mode-changed",
                             G_CALLBACK (photos_application_window_mode_changed),
@@ -1886,6 +1986,9 @@ photos_application_startup (GApplication *application)
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.save-current", save_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.search", search_accels);
   gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.select-all", select_all_accels);
+  gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.zoom-best-fit", zoom_best_fit_accels);
+  gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.zoom-in(-1.0)", zoom_in_accels);
+  gtk_application_set_accels_for_action (GTK_APPLICATION (self), "app.zoom-out(-1.0)", zoom_out_accels);
 
   g_signal_connect_swapped (self->state->item_mngr,
                             "load-finished",
@@ -1903,6 +2006,8 @@ photos_application_dispose (GObject *object)
 {
   PhotosApplication *self = PHOTOS_APPLICATION (object);
 
+  g_assert_null (self->search_provider);
+
   if (self->miners_running != NULL)
     {
       g_list_free_full (self->miners_running, g_object_unref);
@@ -1955,10 +2060,14 @@ photos_application_dispose (GObject *object)
   g_clear_object (&self->search_type_action);
   g_clear_object (&self->sel_all_action);
   g_clear_object (&self->sel_none_action);
+  g_clear_object (&self->selection_mode_action);
   g_clear_object (&self->set_bg_action);
   g_clear_object (&self->set_ss_action);
   g_clear_object (&self->share_action);
   g_clear_object (&self->sharpen_action);
+  g_clear_object (&self->zoom_best_fit_action);
+  g_clear_object (&self->zoom_in_action);
+  g_clear_object (&self->zoom_out_action);
   g_clear_object (&self->shr_pnt_mngr);
   g_clear_object (&self->camera_cache);
   g_clear_object (&self->sel_cntrlr);
@@ -1994,16 +2103,6 @@ photos_application_init (PhotosApplication *self)
 {
   photos_utils_ensure_builtins ();
 
-  self->search_provider = photos_search_provider_new ();
-  g_signal_connect_swapped (self->search_provider,
-                            "activate-result",
-                            G_CALLBACK (photos_application_activate_result),
-                            self);
-  g_signal_connect_swapped (self->search_provider,
-                            "launch-search",
-                            G_CALLBACK (photos_application_launch_search),
-                            self);
-
   self->state = photos_search_context_state_new (PHOTOS_SEARCH_CONTEXT (self));
   self->activation_timestamp = GDK_CURRENT_TIME;
 
