@@ -225,21 +225,6 @@ photos_main_window_handle_key_edit (PhotosMainWindow *self, GdkEventKey *event)
 }
 
 
-static gboolean
-photos_main_window_handle_key_overview (PhotosMainWindow *self, GdkEventKey *event)
-{
-  gboolean handled = FALSE;
-
-  if (photos_selection_controller_get_selection_mode (self->sel_cntrlr) && event->keyval == GDK_KEY_Escape)
-    {
-      photos_selection_controller_set_selection_mode (self->sel_cntrlr, FALSE);
-      handled = TRUE;
-    }
-
-  return handled;
-}
-
-
 static gboolean
 photos_main_window_handle_key_preview (PhotosMainWindow *self, GdkEventKey *event)
 {
@@ -295,6 +280,10 @@ photos_main_window_key_press_event (GtkWidget *widget, GdkEventKey *event)
   switch (mode)
     {
     case PHOTOS_WINDOW_MODE_NONE:
+    case PHOTOS_WINDOW_MODE_COLLECTIONS:
+    case PHOTOS_WINDOW_MODE_FAVORITES:
+    case PHOTOS_WINDOW_MODE_OVERVIEW:
+    case PHOTOS_WINDOW_MODE_SEARCH:
       handled = GDK_EVENT_PROPAGATE;
       break;
 
@@ -306,13 +295,6 @@ photos_main_window_key_press_event (GtkWidget *widget, GdkEventKey *event)
       handled = photos_main_window_handle_key_preview (self, event);
       break;
 
-    case PHOTOS_WINDOW_MODE_COLLECTIONS:
-    case PHOTOS_WINDOW_MODE_FAVORITES:
-    case PHOTOS_WINDOW_MODE_OVERVIEW:
-    case PHOTOS_WINDOW_MODE_SEARCH:
-      handled = photos_main_window_handle_key_overview (self, event);
-      break;
-
     default:
       g_assert_not_reached ();
       break;
@@ -377,8 +359,7 @@ photos_main_window_constructed (GObject *object)
   self->load_next = g_action_map_lookup_action (G_ACTION_MAP (app), "load-next");
   self->load_previous = g_action_map_lookup_action (G_ACTION_MAP (app), "load-previous");
 
-  self->embed = photos_embed_new ();
-  gtk_container_add (GTK_CONTAINER (self), self->embed);
+  gtk_widget_init_template (GTK_WIDGET (self));
 }
 
 
@@ -458,6 +439,9 @@ photos_main_window_class_init (PhotosMainWindowClass *class)
   widget_class->delete_event = photos_main_window_delete_event;
   widget_class->key_press_event = photos_main_window_key_press_event;
   widget_class->window_state_event = photos_main_window_window_state_event;
+
+  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/Photos/main-window.ui");
+  gtk_widget_class_bind_template_child (widget_class, PhotosMainWindow, embed);
 }
 
 
@@ -471,8 +455,6 @@ photos_main_window_new (GtkApplication *application)
                        "height_request", WINDOW_MIN_HEIGHT,
                        "application", application,
                        "title", _(PACKAGE_NAME),
-                       "window-position", GTK_WIN_POS_CENTER,
-                       "show-menubar", FALSE,
                        NULL);
 }
 
