@@ -32,6 +32,12 @@
 #include "photos-search-context.h"
 
 
+struct _PhotosOffsetSearchController
+{
+  PhotosOffsetController parent_instance;
+};
+
+
 G_DEFINE_TYPE (PhotosOffsetSearchController, photos_offset_search_controller, PHOTOS_TYPE_OFFSET_CONTROLLER);
 
 
