@@ -25,7 +25,6 @@
 
 #include <babl/babl.h>
 #include <gegl.h>
-#include <gegl-plugin.h>
 
 #include "photos-enums.h"
 #include "photos-operation-insta-common.h"
@@ -41,11 +40,6 @@ struct _PhotosOperationInstaFilter
   PhotosOperationInstaPreset preset;
 };
 
-struct _PhotosOperationInstaFilterClass
-{
-  GeglOperationMetaClass parent_class;
-};
-
 enum
 {
   PROP_0,
