@@ -24,7 +24,6 @@
 #include <babl/babl.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gegl.h>
-#include <gegl-plugin.h>
 
 #include "photos-operation-insta-hefe-vignette.h"
 
@@ -40,11 +39,6 @@ struct _PhotosOperationInstaHefeVignette
   gdouble y;
 };
 
-struct _PhotosOperationInstaHefeVignetteClass
-{
-  GeglOperationPointRenderClass parent_class;
-};
-
 enum
 {
   PROP_0,
