@@ -18,27 +18,25 @@
  * 02110-1301, USA.
  */
 
+/* Based on code from:
+ *   + GEGL
+ */
+
 #ifndef PHOTOS_OPERATION_SVG_MULTIPLY_H
 #define PHOTOS_OPERATION_SVG_MULTIPLY_H
 
-#include <glib-object.h>
+#include <gegl-plugin.h>
+
+#include "photos-gegl.h"
 
 G_BEGIN_DECLS
 
 #define PHOTOS_TYPE_OPERATION_SVG_MULTIPLY (photos_operation_svg_multiply_get_type ())
-
-#define PHOTOS_OPERATION_SVG_MULTIPLY(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
-   PHOTOS_TYPE_OPERATION_SVG_MULTIPLY, PhotosOperationSvgMultiply))
-
-#define PHOTOS_IS_OPERATION_SVG_MULTIPLY(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
-   PHOTOS_TYPE_OPERATION_SVG_MULTIPLY))
-
-typedef struct _PhotosOperationSvgMultiply      PhotosOperationSvgMultiply;
-typedef struct _PhotosOperationSvgMultiplyClass PhotosOperationSvgMultiplyClass;
-
-GType            photos_operation_svg_multiply_get_type       (void) G_GNUC_CONST;
+G_DECLARE_FINAL_TYPE (PhotosOperationSvgMultiply,
+                      photos_operation_svg_multiply,
+                      PHOTOS,
+                      OPERATION_SVG_MULTIPLY,
+                      GeglOperationPointComposer);
 
 G_END_DECLS
 
