@@ -33,6 +33,7 @@
 #include "photos-icons.h"
 #include "photos-preview-nav-buttons.h"
 #include "photos-search-context.h"
+#include "photos-zoom-controls.h"
 
 
 struct _PhotosPreviewNavButtons
@@ -46,11 +47,15 @@ struct _PhotosPreviewNavButtons
   GtkWidget *overlay;
   GtkWidget *prev_widget;
   GtkWidget *preview_view;
+  GtkWidget *zoom_revealer;
+  GtkWidget *zoom_widget;
   PhotosBaseManager *item_mngr;
   PhotosModeController *mode_cntrlr;
   PhotosWindowMode old_mode;
+  gboolean auto_hide;
   gboolean enable_next;
   gboolean enable_prev;
+  gboolean show_navigation;
   gboolean visible;
   gboolean visible_internal;
   guint auto_hide_id;
@@ -140,11 +145,11 @@ photos_preview_nav_buttons_notify_child_revealed (PhotosPreviewNavButtons *self,
 static void
 photos_preview_nav_buttons_fade_out_button (PhotosPreviewNavButtons *self, GtkWidget *widget)
 {
+  gtk_revealer_set_reveal_child (GTK_REVEALER (widget), FALSE);
   g_signal_connect_swapped (widget,
                             "notify::child-revealed",
                             G_CALLBACK (photos_preview_nav_buttons_notify_child_revealed),
                             self);
-  gtk_revealer_set_reveal_child (GTK_REVEALER (widget), FALSE);
 }
 
 
@@ -192,34 +197,45 @@ static void
 photos_preview_nav_buttons_update_visibility (PhotosPreviewNavButtons *self)
 {
   PhotosBaseItem *item;
-  PhotosBaseItem *next_item;
-  PhotosBaseItem *previous_item;
+  gboolean enable_zoom = TRUE;
 
   item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (self->item_mngr));
+
   if (self->old_mode == PHOTOS_WINDOW_MODE_NONE || item == NULL || !self->visible)
     {
       self->enable_prev = FALSE;
       self->enable_next = FALSE;
-      goto out;
+    }
+  else
+    {
+      PhotosBaseItem *next_item;
+      PhotosBaseItem *previous_item;
+
+      previous_item = photos_preview_nav_buttons_get_previous_item (self);
+      self->enable_prev = previous_item != NULL;
+
+      next_item = photos_preview_nav_buttons_get_next_item (self);
+      self->enable_next = next_item != NULL;
     }
 
-  previous_item = photos_preview_nav_buttons_get_previous_item (self);
-  self->enable_prev = previous_item != NULL;
+  if (item == NULL || !self->visible)
+    enable_zoom = FALSE;
 
-  next_item = photos_preview_nav_buttons_get_next_item (self);
-  self->enable_next = next_item != NULL;
-
- out:
-  if (self->visible_internal && self->enable_next)
+  if (self->visible_internal && self->show_navigation && self->enable_next)
     photos_preview_nav_buttons_fade_in_button (self, self->next_widget);
   else
     photos_preview_nav_buttons_fade_out_button (self, self->next_widget);
 
-  if (self->visible_internal && self->enable_prev)
+  if (self->visible_internal && self->show_navigation && self->enable_prev)
     photos_preview_nav_buttons_fade_in_button (self, self->prev_widget);
   else
     photos_preview_nav_buttons_fade_out_button (self, self->prev_widget);
 
+  if (self->visible_internal && enable_zoom)
+    photos_preview_nav_buttons_fade_in_button (self, self->zoom_revealer);
+  else
+    photos_preview_nav_buttons_fade_out_button (self, self->zoom_revealer);
+
   g_simple_action_set_enabled (G_SIMPLE_ACTION (self->load_next), self->enable_next);
   g_simple_action_set_enabled (G_SIMPLE_ACTION (self->load_previous), self->enable_prev);
 }
@@ -265,7 +281,9 @@ static void
 photos_preview_nav_buttons_queue_auto_hide (PhotosPreviewNavButtons *self)
 {
   photos_preview_nav_buttons_unqueue_auto_hide (self);
-  self->auto_hide_id = g_timeout_add_seconds (2, (GSourceFunc) photos_preview_nav_buttons_auto_hide, self);
+
+  if (self->auto_hide)
+    self->auto_hide_id = g_timeout_add_seconds (2, (GSourceFunc) photos_preview_nav_buttons_auto_hide, self);
 }
 
 
@@ -283,7 +301,10 @@ photos_preview_nav_buttons_motion_notify_timeout (PhotosPreviewNavButtons *self)
   self->motion_id = 0;
   self->visible_internal = TRUE;
   photos_preview_nav_buttons_update_visibility (self);
-  photos_preview_nav_buttons_queue_auto_hide (self);
+
+  if (!photos_zoom_controls_get_hover (PHOTOS_ZOOM_CONTROLS (self->zoom_widget)))
+    photos_preview_nav_buttons_queue_auto_hide (self);
+
   return G_SOURCE_REMOVE;
 }
 
@@ -297,6 +318,9 @@ photos_preview_nav_buttons_motion_notify (PhotosPreviewNavButtons *self, GdkEven
   if (self->motion_id != 0)
     return FALSE;
 
+  if (!self->auto_hide)
+    return FALSE;
+
   device = gdk_event_get_source_device ((GdkEvent *) event);
   input_source = gdk_device_get_source (device);
   if (input_source == GDK_SOURCE_TOUCHSCREEN)
@@ -327,9 +351,13 @@ photos_preview_nav_buttons_multi_press_end (PhotosPreviewNavButtons *self, GdkEv
     return;
 
   gtk_gesture_set_state (GTK_GESTURE (self->tap_gesture), GTK_EVENT_SEQUENCE_CLAIMED);
-  self->visible_internal = !self->visible_internal;
-  photos_preview_nav_buttons_unqueue_auto_hide (self);
-  photos_preview_nav_buttons_update_visibility (self);
+
+  if (self->auto_hide)
+    {
+      self->visible_internal = !self->visible_internal;
+      photos_preview_nav_buttons_unqueue_auto_hide (self);
+      photos_preview_nav_buttons_update_visibility (self);
+    }
 }
 
 
@@ -377,6 +405,16 @@ photos_preview_nav_buttons_previous (PhotosPreviewNavButtons *self)
 }
 
 
+static void
+photos_preview_nav_buttons_zoom_notify_hover (PhotosPreviewNavButtons *self)
+{
+  if (photos_zoom_controls_get_hover (PHOTOS_ZOOM_CONTROLS (self->zoom_widget)))
+    photos_preview_nav_buttons_unqueue_auto_hide (self);
+  else
+    photos_preview_nav_buttons_leave_notify (self);
+}
+
+
 static void
 photos_preview_nav_buttons_dispose (GObject *object)
 {
@@ -388,6 +426,8 @@ photos_preview_nav_buttons_dispose (GObject *object)
   g_clear_object (&self->tap_gesture);
   g_clear_object (&self->next_widget);
   g_clear_object (&self->prev_widget);
+  g_clear_object (&self->zoom_revealer);
+  g_clear_object (&self->zoom_widget);
   g_clear_object (&self->item_mngr);
   g_clear_object (&self->mode_cntrlr);
 
@@ -405,6 +445,25 @@ photos_preview_nav_buttons_constructed (GObject *object)
 
   G_OBJECT_CLASS (photos_preview_nav_buttons_parent_class)->constructed (object);
 
+  self->zoom_revealer = g_object_ref_sink (gtk_revealer_new ());
+  gtk_widget_set_halign (self->zoom_revealer, GTK_ALIGN_END);
+  gtk_widget_set_margin_bottom (self->zoom_revealer, 30);
+  gtk_widget_set_margin_end (self->zoom_revealer, 30);
+  gtk_widget_set_margin_start (self->zoom_revealer, 30);
+  gtk_widget_set_margin_top (self->zoom_revealer, 30);
+  gtk_widget_set_valign (self->zoom_revealer, GTK_ALIGN_END);
+  gtk_revealer_set_transition_type (GTK_REVEALER (self->zoom_revealer), GTK_REVEALER_TRANSITION_TYPE_CROSSFADE);
+  gtk_overlay_add_overlay (GTK_OVERLAY (self->overlay), self->zoom_revealer);
+
+  self->zoom_widget = g_object_ref_sink (photos_zoom_controls_new ());
+  context = gtk_widget_get_style_context (self->zoom_widget);
+  gtk_style_context_add_class (context, "osd");
+  gtk_container_add (GTK_CONTAINER (self->zoom_revealer), self->zoom_widget);
+  g_signal_connect_swapped (self->zoom_widget,
+                            "notify::hover",
+                            G_CALLBACK (photos_preview_nav_buttons_zoom_notify_hover),
+                            self);
+
   self->prev_widget = g_object_ref_sink (gtk_revealer_new ());
   gtk_widget_set_halign (self->prev_widget, GTK_ALIGN_START);
   gtk_widget_set_margin_start (self->prev_widget, 30);
@@ -463,6 +522,7 @@ photos_preview_nav_buttons_constructed (GObject *object)
                             self);
 
   self->long_press_gesture = gtk_gesture_long_press_new (self->preview_view);
+  gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (self->long_press_gesture), GTK_PHASE_CAPTURE);
   gtk_gesture_single_set_touch_only (GTK_GESTURE_SINGLE (self->long_press_gesture), TRUE);
   g_signal_connect_swapped (self->long_press_gesture,
                             "pressed",
@@ -470,6 +530,7 @@ photos_preview_nav_buttons_constructed (GObject *object)
                             self);
 
   self->tap_gesture = gtk_gesture_multi_press_new (self->preview_view);
+  gtk_event_controller_set_propagation_phase (GTK_EVENT_CONTROLLER (self->tap_gesture), GTK_PHASE_CAPTURE);
   gtk_gesture_single_set_touch_only (GTK_GESTURE_SINGLE (self->tap_gesture), TRUE);
   gtk_gesture_group (self->long_press_gesture, self->tap_gesture);
   g_signal_connect_swapped (self->tap_gesture,
@@ -607,12 +668,33 @@ photos_preview_nav_buttons_new (PhotosPreviewView *preview_view, GtkOverlay *ove
 void
 photos_preview_nav_buttons_hide (PhotosPreviewNavButtons *self)
 {
+  g_return_if_fail (PHOTOS_IS_PREVIEW_NAV_BUTTONS (self));
+
   self->visible = FALSE;
   self->visible_internal = FALSE;
   photos_preview_nav_buttons_update_visibility (self);
 }
 
 
+void
+photos_preview_nav_buttons_set_auto_hide (PhotosPreviewNavButtons *self, gboolean auto_hide)
+{
+  g_return_if_fail (PHOTOS_IS_PREVIEW_NAV_BUTTONS (self));
+
+  if (self->auto_hide == auto_hide)
+    return;
+
+  self->auto_hide = auto_hide;
+  self->visible_internal = self->visible;
+  photos_preview_nav_buttons_update_visibility (self);
+
+  if (self->auto_hide)
+    photos_preview_nav_buttons_queue_auto_hide (self);
+  else
+    photos_preview_nav_buttons_unqueue_auto_hide (self);
+}
+
+
 void
 photos_preview_nav_buttons_set_mode (PhotosPreviewNavButtons *self, PhotosWindowMode old_mode)
 {
@@ -624,9 +706,24 @@ photos_preview_nav_buttons_set_mode (PhotosPreviewNavButtons *self, PhotosWindow
 }
 
 
+void
+photos_preview_nav_buttons_set_show_navigation (PhotosPreviewNavButtons *self, gboolean show_navigation)
+{
+  g_return_if_fail (PHOTOS_IS_PREVIEW_NAV_BUTTONS (self));
+
+  if (self->show_navigation == show_navigation)
+    return;
+
+  self->show_navigation = show_navigation;
+  photos_preview_nav_buttons_update_visibility (self);
+}
+
+
 void
 photos_preview_nav_buttons_show (PhotosPreviewNavButtons *self)
 {
+  g_return_if_fail (PHOTOS_IS_PREVIEW_NAV_BUTTONS (self));
+
   self->visible = TRUE;
   self->visible_internal = TRUE;
   photos_preview_nav_buttons_update_visibility (self);
